#include <iostream>
#include <string>
#include <vector>

#include "TApplication.h"
#include "TGraph.h"
#include "Garfield/MediumMagboltz.hh"
#include "Garfield/MediumConductor.hh"
#include "Garfield/GeometryRoot.hh"
#include "Garfield/GeometrySimple.hh"
#include "Garfield/SolidBox.hh"
#include "Garfield/SolidWire.hh"
#include "Garfield/ComponentAnalyticField.hh"
#include "Garfield/Sensor.hh"
#include "Garfield/TrackHeed.hh"
#include "Garfield/TrackPAI.hh"
#include "Garfield/DriftLineRKF.hh"
#include "Garfield/AvalancheMC.hh"
#include "Garfield/AvalancheMicroscopic.hh"
#include "Garfield/ViewGeometry.hh"
#include "Garfield/ViewField.hh"
#include "Garfield/ViewCell.hh"
#include "Garfield/ViewDrift.hh"
#include "Garfield/ViewSignal.hh"

const double pressure = 750.;
const double temperature = 293.15;
const double d_sense = 0.0025; //cm
const double d_field = 0.0126; //cm

using namespace Garfield;

int main(int argc, char** argv){

	TApplication app("app", &argc, argv);

	MediumMagboltz gas;
	gas.LoadGasFile("../He_90_iC4H10_10_760Torr_293.15K_10emin_to_500000emax.gas");
	gas.LoadIonMobility("../IonMobility_He+_He.txt");
	gas.Initialise(true);

	GeometrySimple geo;
	ComponentAnalyticField wire;

	wire.AddWire(-0.84, -0.8, d_field, 0, "field");
	wire.AddWire(0., -0.8, d_field, 0, "field");
	wire.AddWire(0.84, -0.8, d_field, 0, "field");
	wire.AddWire(-0.84, 0., d_field, 0, "field");
	wire.AddWire(0.84, 0., d_field, 0, "field");
	wire.AddWire(-0.84, 0.8, d_field, 0, "field");
	wire.AddWire(0., 0.8, d_field, 0, "field");
	wire.AddWire(0.84, 0.8, d_field, 0, "field");
	wire.AddWire(0., 0., d_sense, 1800, "sense");

	wire.SetMedium(&gas);
	wire.SetMagneticField(0, 0, 0);
        wire.AddReadout("sense");

	const double tMin = 0.;
	const double tMax = 1000.;
	const double tStep = 0.2;
	const int nSteps = (int)(tMax-tMin)/tStep;
	Sensor sensor;
	sensor.AddComponent(&wire);
	sensor.SetArea(-1., -1., -80., 1., 1., 80.);
	sensor.AddElectrode(&wire, "sense");
	sensor.SetTimeWindow(tMin, tStep, nSteps);

	TrackHeed track;
	AvalancheMC drift;
	ViewCell cellView;
	ViewField fieldView;
	ViewDrift driftView;
	ViewSignal signalView;

	signalView.SetSensor(&sensor);

	track.SetSensor(&sensor);
	track.SetParticle("muon");
	track.SetMomentum(1e9);
	track.EnableDeltaElectronTransport();

	double maxrange = 0.;
	double rforstraight = 0.;
	double stepstraight = 0.;
	double stepcurved = 0.;
	track.EnableElectricField();
	track.EnableMagneticField();                
	track.GetSteppingLimits(maxrange, rforstraight, stepstraight, stepcurved);
	std::cout<<maxrange<<" "<<rforstraight<<" "<<stepstraight<<" "<<stepcurved<<std::endl;
	maxrange = 0.01;
	stepcurved = 0.02;
	track.SetSteppingLimits(maxrange, rforstraight, stepstraight, stepcurved);

	drift.SetSensor(&sensor);
	drift.EnableSignalCalculation();

	track.EnablePlotting(&driftView);
	drift.EnablePlotting(&driftView);

	track.NewTrack(-0.5, 1., 0., 0., 0., -1., 0.);
        double xc, yc, zc, tc, ec, extra;
	int nc;
	while(track.GetCluster(xc, yc, zc, tc, nc, ec, extra)){
		double del_x, del_y, del_z, del_t, ke, dummy;
		for(int i=0; i<nc; i++){
			track.GetElectron(i, del_x, del_y, del_z, del_t, ke, dummy, dummy, dummy);
			drift.DriftElectron(del_x, del_y, del_z, del_t);
			size_t ne = drift.GetNumberOfElectronEndpoints();
			std::cout<<ne<<" electrons!!!!!"<<std::endl;
		}
	}

        cellView.EnableWireMarkers();
	cellView.SetComponent(&wire);
	cellView.SetArea(-1., -1., -80., 1., 1., 80.);
	cellView.Plot2d();
	fieldView.SetComponent(&wire);
	cellView.SetArea(-1., -1., -80., 1., 1., 80.);
	fieldView.SetSensor(&sensor);
	fieldView.SetNumberOfContours(50);
	fieldView.PlotContour("p");
	driftView.Plot(1, 0);
//        signalView.PlotSignal("sense");

	app.Run();

	return 0;
}
